
import OFS
from OFS import SimpleItem
import Globals
from Globals import DTMLFile
import AccessControl.Role

import urllib
import string
import re

from email.Utils import *

class AddressBook(OFS.Folder.Folder) :
    "an AddressBook Folder"

    
    meta_type = 'Emil_AddressBook'

    ad_options = ( {'label': 'view',   'action': 'index_html',},
		   {'label': 'import',   'action': 'import_html',},		       
		   )
    
    manage_options = ( ad_options +
		       OFS.SimpleItem.Item.manage_options
		       )

    __ac_permissions__=(
	('View management screens',['manage_editForm',
				    'manage_editAction',
				    'manage_importAddressBook',
				    'manage_write_email_to_selected'], ('Owner',)),
	('View',  	           ['index_html','import_html'],			    ('Owner',)),     
        )     
    
    index_html         = DTMLFile('dtml/adrbook/addressbook_index_html'      , globals())
    import_html        = DTMLFile('dtml/adrbook/addressbook_import_html'      , globals())
    
    def __init__(self,id,title) :
	self.id    = id
	self.title = title
	self.groups={}
	
    def manage_main(trueself,self,REQUEST=None,update_menu=0,cb_dataValid=0,manage_tabs_message=None):
	""" wrapper around manage_main """
	return self.index_html(REQUEST,update_menu,cb_dataValid,manage_tabs_message)


    def get_addresses(self) :
	""" returns list of address objects """
	return self.objectItems('Emil_address')

    def manage_importAddressBook(self,file_type, REQUEST=None) :
	""" import data from external email client """

	if file_type != 'kmail' :
	    message = "import from %s not supported" % file_type
	    return self.import_html(REQUEST,manage_tabs_message=message)

	file = REQUEST.form['attached_file']
	
	for line in string.split(file,"\n") :
	    if line == "" : continue
	    if re.match("^#",line) :
		# commented line, skip it
		continue
	    # check for correct format of line
	    # very simple check hm.
	    match = re.match(".*@.*",line)
	    if match == None :		
		print "can not parse line >%s<" % line
		continue

	    # some problems with parseaddr require this:
	    try:
		spl = string.split(line,"<")
		realname = spl[0]
		email = spl[1]
		r, address  = parseaddr(email)
	    except:
		# something stupid happened, catch this case...
		# user will have to format this address entry himself...
		realname = line
	    #print line
	    #print ">%s<,>%s<" % (realname, address)
	    manage_addAddress(self,firstname=realname,email=address)
	    #return self.index_html(REQUEST)
	message = "addressbook successfully imported"
	return self.index_html(REQUEST,manage_tabs_message=message)

    def manage_write_email_to_selected(self,ids=[],REQUEST=None,RESPONSE=None) :
	""" send email to selected addresses from adress book """

	to = ""
	c = 0
	for id in ids :
	    c = c + 1
	    obj = getattr(self,id)
	    adr = obj.__str__()
	    if c > 1 :
		to += ",\n"
	    to += adr
	
	if RESPONSE != None :
	    # return method of higher level email folder
	    url = REQUEST['URL2']
	    path = urllib.urlencode({'To':to})
	    npath = url + '/folder_write_mail_html?' + path	    
	    return REQUEST.RESPONSE.redirect(npath)
	return self.folder_write_mail_html(To=to)
    

    def add_group(self,name,REQUEST) :
	""" add groups """
	try:
	    self.groups[name] = 1
	except :
	    # catch for old versions
	    self.groups = {}
	    self.groups[name] = 1 
	    self._p_changed = 1
	    
	self._p_changed = 1
	message = "added group %s" % name
	if REQUEST != None :
	    return self.index_html(REQUEST,manage_tabs_message=message)

    def get_groups(self):
	""" returns groups """
	try:
	    g = self.groups.keys()
	except:
	    # catch for old versions
	    self.groups = {}
	    self._p_changed = 1
	    
	return self.groups.keys()

    def get_addresses_from_group(self,group):
	""" returns addresses from group """
	list = self.objectItems('Emil_address')
	
	grouplst = []
	for tup in list :
	    add = tup[1]
	    if group == 'all' :
		grouplst.append(tup)
	
	    if group in add.get_address_groups() :
		grouplst.append(tup)
	return grouplst
    
    def delete_group(self,name,REQUEST=None):
	"""delete group number ..."""

	del self.groups[name]
	self._p_changed = 1
	message = "deleted group %s" % name
	if REQUEST != None :
	    return self.index_html(REQUEST,manage_tabs_message=message)

    def mail_group(self,group,REQUEST=None) :
	""" write email to member of this group """
	lst = self.get_addresses_from_group(group)

	ids = []
	for a in lst :
	    ids.append(a[0])
	
	return self.manage_write_email_to_selected(ids=ids,REQUEST=REQUEST)
	    
	
    def address_actions(self,ids=None,command=None,REQUEST=None,RESPONSE=None) :
	""" do something with messages """
	
	message = command
	if ids == None :
	    message = "no selection" 
	    if REQUEST != None :
		return self.index_html(REQUEST,manage_tabs_message=message)
	    else :
		return
	if command == "write"  :
	    return self.manage_write_email_to_selected(ids,REQUEST,RESPONSE)
		
	elif command[:4] == 'add_' :
	    group = command[4:]
	   
	    for id in ids:
		obj = getattr(self,id)
		obj.add_group(group)
	    if REQUEST != None :
		return self.index_html(REQUEST,manage_tabs_message=message,group=group)

	else :
	    raise "UnknownCommand",'unknown command %s' % command
	    
	if REQUEST != None :
	    return self.index_html(REQUEST,manage_tabs_message=message)

	    
manage_addAddressbookForm = DTMLFile('dtml/adrbook/addressbook_Add',globals())
	
def manage_addAddressBook(self,id,title,REQUEST=None) :
    """ add an address book"""
    
    self._setObject(id,AddressBook(id,title))
    if REQUEST is not None: return self.manage_main(self,REQUEST)

    

class Address(SimpleItem.SimpleItem):
    """ an address entry """

    meta_type = "Emil_address"
    
    manage_options = (
	{'label': 'View',          'action': 'index_html'},
	)

    __ac_permissions__=(
	('View', ['index_html',], ('Owner',)),
	('View management screens',['manage_editForm',], ('Owner',)),
        ) 

    index_html        = DTMLFile('dtml/adrbook/address_index_html'    , globals())
    manage_editForm   = DTMLFile('dtml/adrbook/address_editForm_html' , globals())
    #manage_editForm   = index_html
    
    def __init__(self,id,firstname,lastname,nickname,email) :
	""" initialize address """
	self.id        = id 
	self.firstname = firstname
	self.lastname  = lastname
	self.nickname  = nickname
	self.email     = email
	self.address_groups = {}
	
    def address_editAddressAction(self,firstname,lastname,nickname,email, REQUEST=None,RESPONSE=None):
	""" edit address"""
	self.firstname = firstname
	self.lastname  = lastname
	self.nickname  = nickname
	self.email     = email
	self._p_changed = 1
	self.update_address_table()
	message = "properties have been changed"
	return self.index_html(self, REQUEST,manage_tabs_message=message)

    def html_display(self,field) :
	""" html representation of email"""
	name = "%s %s" % (self.firstname,self.lastname)
	if name == "" :
	    return self.email
	else :
	    str = name +" <%s>" % self.email
	    return urllib.urlencode({field:str})
	    
	
    def __str__(self):
	""" string representation of message """
	name = "%s %s" % (self.firstname,self.lastname)
	if name == "" :
	    str = self.email
	else :
	    str = "%s <%s>" % (name,self.email)
	return str

    def get_nickname(self):
	return self.nickname

    def get_email(self):
	return self.email

    def get_address_groups(self):
	""" returns the groups of this address """
	try:
	    g = self.address_groups.keys()
	except:
	    # catch for old versions
	    print "catch 1"
	    self.address_groups = {}
	    self._p_changed = 1
	    g = self.address_groups.keys()
	    
	# check if this group is still existing,
	# if not, remove it
	for gr in g :
	    if not gr in self.get_groups():
		self.remove_from_group(gr)
	    
	return self.address_groups.keys()

    def add_group(self,name,REQUEST=None) :
	""" add group to this address"""
	try:
	    g = self.address_groups.keys()
	except:
	    # catch for old versions
	    print "catch!2"
	    self.address_groups = {}
	    self._p_changed = 1
	    
	self.address_groups[name] = 1
	self._p_changed = 1
	message = "added to group %s" % name
	if REQUEST != None :
	    return self.index_html(REQUEST,manage_tabs_message=message)
	
    def remove_from_group(self,name,REQUEST=None) :
	""" delete group """
	del self.address_groups[name]
	self._p_changed = 1
	message = "removed from group %s" % name
	if REQUEST != None :
	    return self.index_html(REQUEST,manage_tabs_message=message)

    def get_new_groups(self) :
	""" get groups that this address does not belong to """
	ret = []
	for group in self.get_groups() :
	    if not group in self.get_address_groups() :
		ret.append(group)
	return ret
    
manage_addAddressForm = DTMLFile('dtml/adrbook/address_manageAddForm_html',globals())
	
def manage_addAddress(self,firstname='',lastname='',nickname='',email='',REQUEST=None) :
    """ add an address """

    id = self.create_unique_id()	
  
    self._setObject(id,Address(id,firstname,lastname,nickname,email))

    self.update_address_table()	
    if REQUEST is not None:
	url = REQUEST['URL3']
	return REQUEST.RESPONSE.redirect(url)
    return self.index_html(self,REQUEST)

