"""
EMIL EMAIL CLIENT
Copyright (C) 2002  Andreas Prlic
andorsch@gmx.at

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


"""
#import Globals
import OFS
from OFS import SimpleItem

import Globals
from Globals import DTMLFile
#from Globals import Persistent
#import Acquisition                	# Various acquisition types. 
import AccessControl.Role         	# Zope Security management.
#from  Products.ZCatalog.CatalogAwareness import CatalogAware

import urllib
from email_message import Email

from filters import *

class email_folder(OFS.Folder.Folder) :

    """ the general email folder """

    meta_type = 'Email_Folder'
    folder_options = (	{'label': 'View_Messages',      'action': 'folder_view_html',},
			{'label': 'Folder_Properties',  'action': 'folder_manage_edit_Form',},
			)
		     
    #(OFS.ObjectManager.ObjectManager.manage_options[0],)+
    manage_options = (folder_options +
		      AccessControl.Role.RoleManager.manage_options+
		      OFS.SimpleItem.Item.manage_options+
		      OFS.FindSupport.FindSupport.manage_options
		      )
    
    
    __ac_permissions__=(
	('View management screens',['folder_manage_edit_Form',
				    'manage_editAction'], ('Owner',)),
	('View',  	           ['folder_index_html',
				    'folder_view_html',
				    'index_html','manage_main'], ('Owner',)),
	('Emil_WAP_access',        ['folder_wml',],	 	('Owner',)),
        )     

    weekdays = ("Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday",)
    

    
    
    folder_view_html         = DTMLFile('dtml/folder_view_html'        , globals())
    folder_manage_edit_Form  = DTMLFile('dtml/folder_manage_edit_Form' , globals())
    #manage_renameForm        = Globals.DTMLFile('dtml/folder_renameForm', globals())

    folder_main              = DTMLFile('dtml/folder_main', globals())

    folder_wml               = DTMLFile('dtml/wap/folder_wml'          , globals())
    message_actions          = DTMLFile('dtml/message_actions'         , globals())
    #index_html = OFS.Folder.Folder.manage_workspace
    #index_html       = folder_view_html
    #manage_main      = index_html

  

    def manage_main(trueself,self,REQUEST=None,update_menu=0,cb_dataValid=0,manage_tabs_message=None):
	""" wrapper around manage_main """
	return self.folder_index_html(REQUEST,update_menu,cb_dataValid,manage_tabs_message)

    def index_html(self,REQUEST,RESPONSE):
	""" wrapper around index_html """
	return self.folder_view_html(REQUEST)

    def __init__ (self, id, title ) :
	""" inits the Folder with default values """
	self.id       = id
	self.title    = title
	self.sms_days = {}
	self.sms_gateway_id = None

    def manage_editAction(self, title,sms_gateway_id=None,REQUEST=None) :
	""" change the product values """

	self.title = title
	self.sms_days = {}
	for day in self.weekdays :
	    if REQUEST.has_key(day) :
		self.sms_days[day] = 1

	self.sms_gateway_id=sms_gateway_id
	
	self._p_changed = 1
	message = "properties have been changed"
	return self.folder_manage_edit_Form(self, REQUEST,manage_tabs_message=message)
    
    def if_sms_on_day(self,day) :

	if self.sms_days.has_key(day) :
	    return 1
	else :
	    return 0

   
    def get_folder_stats(self) :
	"""returns stats for this folder"""

	ret = []
	emails = self.objectValues('Email')
	unread_count = 0
	for email in emails :
	    if email.status == 'unread' :
		unread_count = unread_count + 1

	fo = ('folder',self.id)
	to = ('total',len(emails))
	un = ('unread',unread_count)
	ret.append(fo)
	ret.append(to)
	ret.append(un)
	return ret

    def get_sub_stats(self,start_depth):
	""" returns statistics of subfolders"""

	maxdepth = start_depth
	dict = {}
	ret = self.get_folder_stats()
	dict['id']     = ret[0][1]
	dict['total']  = ret[1][1]
	dict['unread'] = ret[2][1]
	dict['sublist'] = []
	for folder in self.objectValues('Email_Folder') :
	    data,depth = folder.get_sub_stats(start_depth+1)
	    dict['sublist'].append(data)
	    if depth > maxdepth :
		maxdepth = depth
	return dict,maxdepth

    def check_send_sms(self,email) :
	from DateTime import DateTime    
	today = DateTime().strftime('%A')
	if self.if_sms_on_day(today) :
	    self.send_sms(email)

    def send_sms(self,email):
		
	body = email.get_body()
	# add first few lines of email ...
	sms_body = body[:255-len(email['Subject'])]
	send_body = "%s:\n%s,\n%s" % (email['From'],email['Subject'],sms_body)
	
	sms_gateway = getattr(self.sms_folder,self.sms_gateway_id)
	sms_gateway.send_SMSAction(mesg=send_body)


    def get_emails(self) :
	" return list of emails "
	"return emails in current folder"
	return self.objectItems('Email')


    def manage_addEmail(self,id,title,size,uid) :
	" add an email"	
	self._setObject(id,Email(id,title,size,uid))

    def delete_message(self,id, REQUEST=None):
	""" delete a message """
	self.manage_delObjects(id,REQUEST=REQUEST)
	message = "message has been deleted"

	if REQUEST is not None :
	    return self.folder_view_html(self, REQUEST,manage_tabs_message=message)
	

    def get_email(self,id,coord):
	""" returns the email after the next, sorted by id """
	emails = self.get_emails()
	c = -1
	pos = None
	for mail in emails :
	    m_id = mail[0]
	    c = c + 1
	    if m_id == id :
		pos = c
		break
	
	if pos + coord  >= len(emails) :
	    return None

	mail = emails[pos+coord][1]

	if pos + coord <= 0 :
	    return None
	
	if pos != None :
	    return [ mail ]
	else:
	    return None

    def get_folders(self):
	""" returns the Email_Folder subfolders """
	return self.objectItems('Email_Folder')

    def get_folder_id(self) :
	""" returns id of this folder """
	return self.id
    
    def get_folder_ids(self,parent_id):
	""" returns the subfolder ids """
	if parent_id != '' :
	    parent_id = parent_id + "/"
	    
	path = parent_id + self.id
	ret = [path,]
	for folder in self.objectValues('Email_Folder') :
	    lst = folder.get_folder_ids(path)
	    for f in lst :
		ret.append(f)
	return ret

    def email_actions(self,ids, command=None, REQUEST=None, RESPONSE=None) :
	""" do for these emails ... """

	for id in ids :
	    p = getattr(self,id)
	    p.email_action(command,REQUEST)

	l = len(ids)
	if command == "delete" :
	    message = "<b>%s</b> message(s) deleted" % l
	elif command[0:5] == "move_" :
	    end_folder = command[5:]
	    message = "<b>%s</b> message moved to <b>%s</b>" % (l,end_folder)	    
	elif command == "spamdelete" :
	    message = "<b>%s</b> message(s) added to spam list and deleted" % l
	elif command == "spam" :
	    message = "<b>%s</b> message(s) added to spam list" % l
	else :
	    raise "Unkown_Email_Action" , command

	if REQUEST is not None :
	    m = urllib.urlencode({'manage_tabs_message':message})
	    url = REQUEST['URL1'] + "?%s" % m
	    return REQUEST.RESPONSE.redirect(url)

    def apply_filters(self,REQUEST=None) :
	""" apply filter rules on this folder """

	filter_manager = FilterManager(self.filter_rules,self)
	for email in self.objectValues('Email') :
	    filter_manager.apply(email)

	message = "applyed filter rules on this folder"	
	return self.manage_main(self, REQUEST,manage_tabs_message=message)
	
	    
    def add_DTML(obj,id,title,file):
	path = Globals.package_home(globals())
	f    = open( path + '/' + file + '.dtml')     
	file = f.read()     
	f.close()     
	obj.manage_addDTMLMethod(id,title,file)     
	return getattr(obj,id)


    
manage_addEmail_Folder_Form = DTMLFile('dtml/folder_manage_addEmail_Folder_Form', globals()) 

def manage_addEmail_Folder_Action(self,id,title,REQUEST=None) :
    " add an email folder"	
    self._setObject(id,email_folder(id,title))
    
    if REQUEST is not None:
	return self.manage_main(self, REQUEST)
     


