"""
EMIL EMAIL CLIENT
Copyright (C) 2002  Andreas Prlic
andorsch@gmx.at

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# new in version 0.3 :
	use the python email module (http://www.python.org/doc/lib/module-email.html)

"""


import re
import string

import Globals
import OFS
from OFS import SimpleItem
from Globals import DTMLFile
from DateTime import DateTime    # to create send_mail time

from  Products.ZCatalog.CatalogAwareness import CatalogAware


import email  
from email.Message import Message as _Message
from email import Encoders
from email.MIMEBase import MIMEBase # to add attachments ...
from email.Utils import *
from email.Header import *
import urllib  

import email
#print "email library version: %s" % email.__version__

class Email( CatalogAware,_Message,SimpleItem.SimpleItem  ):
    "an email message"

    meta_type = "Email"
    manage_options = (
	{'label': 'View',          'action': 'index_html'},	
	{'label': 'Reply',         'action': 'reply_mail'},
	{'label': 'Forward',       'action': 'forward_mail'},
	{'label': 'Show_Header',   'action': 'show_header'},
	)

    __ac_permissions__=(
	('View management screens',['index_html',
				    'email_show_header_html'],  ('Owner',)),
	('Emil_WAP_access',        ['email_wml',],	 	('Owner',)),
	)

    index_html       = DTMLFile('dtml/email_index_html'      , globals())
    email_show_header_html  = DTMLFile('dtml/email_show_header_html' , globals())
    email_wml        = DTMLFile('dtml/wap/email_wml' , globals())
    
    def __init__(self,id,title,size,uid) :
	"initialize an email object"
	
	
    	self.id      = id
	self.title   = title
	self.message_str = ""
	
	self.size    = int(size)
	
	# uid = unique id from email server for this message
	# it is used to keep distinguish read / unread messages from server	
	self.uid     = uid

	# at some point status should be displayed for every email
	self.possible_status = ['read', 'unread', \
				'replied', 'forwarded' , \
				'sent', 'queued']
	self.status  = 'unread'


	# donnot knwo why the following line does not work:
	#   _Message.__init__(self)
	# it produces an error:
	# Error Type: TypeError
	# Error Value: unbound method __init__() must be called with instance as first argument
	# but self should be the instance ?? no ?

	# hack around this problem :
	# copy _Message.__init__(self)
	self._headers = []
	self._charset = None
        self._unixfrom = None
        self._payload = None
	# Defaults for multipart messages
        self.preamble = self.epilogue = None

	

    def email_action(self, command=None, REQUEST=None) :
	""" do something with email
	called from index_html
	- possible actions for this message """
	
	if command == "delete" :
	    # delete email
	    self.delete_message(self.id,REQUEST)
	    
	elif command[0:5] == "move_" :
	    endfolder = command[5:]
	else :
	    print " unknown action %s" % command

	if REQUEST is not None :
	    url = REQUEST['URL2']
	    return REQUEST.RESPONSE.redirect(url)


	
    def from_server_list(self,message_lst):
	" build up the message from a server_lst "
	#print message_lst
    	message_str = string.join(message_lst,'\n')
	# for debug reasons store message string	
	self.message_str = message_str

	#print message_str
	
	# correctbug if no subject is found ..
	self['Subject'] = ''
	self['From']    = ''
	self['Date']    = ''
	self['to']      = ''
	
	# creates a new object
	try:	   
	    email_obj  = email.message_from_string(message_str)
	except:
	   
	    # stupid problem with this message

	    # write to file	    
	    #mailFile=open('/tmp/mail.txt',"w")	    
	    #mailFile.write(message_str)
	    #mailFile.close()

	    # handle output
	    import sys
	    import traceback
	    del self['Subject']
	    self['Subject'] = "misformated email" 
	    self.title = "%s %s" % (sys.exc_type, sys.exc_value)
	    info = sys.exc_info()
	    tb = info[2]
	    log = traceback.format_exception(sys.exc_type, sys.exc_value,tb)
	    str = message_str
	    str = str + "\n\nTHE FOLLOWIN ERROR APPEARED DURING PARSING OF THIS MESSAGE:\n"
	    str = str + string.join(log)
	    self.set_payload(str)
	    return

	# set default encoding
	ch =  self.get_charsets()	
	if ch[0] == None :
	    ch = ['latin-1']
	    
	# copy all data  into this object
	# and decode correctly 
	for key in email_obj.keys() :
	    del self[key]
	    t = ""
	    lst = decode_header(email_obj[key])
	    for l in lst :
		if l[1] != None :		    
		    try:
			# for some strange characters this does not work. hm.
			# example: l = ('Petrovi\xc4\x87', 'utf-8')
			# catch this.
			u = unicode(l[0],l[1])
			t =  t + u.encode(ch[0])
		    except :
			
			t = t + l[0]
		else :
		    t = t + l[0]
	    
	    self[key] = t
    
	payload = email_obj.get_payload()
	
	# if email consists of only one attachment that is the message,
	# an instance is returned, not a list or a string
	# catch this condition
	if type(payload) !=  type('string') and type(payload) != type(['list',]) :
	    payload = payload.get_payload(decode=1)
	
	self.set_payload(payload)
	self.title = " %s %s" % (self['From'],self['Subject'])

    def __getattr__(self, name):
	'''get an attribute from the object'''

	if string.lower(name) == 'body' :
	    return self.msg_body()
	
	txt = self.get(name)

	if txt != None :	    
	    return txt
	return _Message.__getattr__(self, name)	


    def show_header(self):
	""" displays the header of an email,
	sets  header and returns email_show_header_html"""

	txt = ""
	
	for part in self.walk() :
	    for item in part.items():
		key   = item[0]
		value = item[1]
		#txt = txt + "%s\t%s\n" % (key,value)
		txt = txt + key + "\t"
		txt = txt + value + "\n"
	    txt = txt + "\n"
	txt = self.highlight_links(txt)
	return self.email_show_header_html(header=txt)


    # ATTACHMENT PART

    # special kind of attachment:
    # if type of message is application/octet-stream
    # return this as an attachment

    def has_attachements(self) :
	" returns 1 if the email has attachements "
	if self.is_multipart() :
	    return 1
	else :
	    type = self.get_type()
	    if type == 'application/octet-stream' :
		return 1
	    return 0

    def get_loads(self, message,number=None) :
	""" recursively goes through all payloads and builds up a list of them.
	needed to convert the possible tree organization of the attachments into a
	linear organization """
	lst = []
	
	msgs = message.get_payload()
	type = message.get_type()
	if type == 'application/octet-stream' :
	    lst.append(msgs)
	    return lst
	
	for msg in msgs :
	    if msg.is_multipart() :
		loads = self.get_loads(msg)
		lst = lst + loads
	    else :
		lst.append(msg)
	if number != None :
	    return lst[number]
	return lst
	    
    def number_attachments(self):
	""" returns the number of attachements, 0 if there are no """
	
	if self.has_attachements() :
	    #pay = self.get_payload()
	    type = self.get_type()
	    # special case
	    if type == 'application/octet-stream' :
		return 1
	    pay = self.get_loads(self)	    
	    l = len(pay) - 1
	    return l
		
	return 0

    def get_attachements(self):
	" returns a list describing the attachements "
	ret = []

	if not self.is_multipart()  :
	    type = self.get_type()
	    if type == 'application/octet-stream' :
		counter = 1
		filename = self.get_filename()
		lst = ( ('counter', counter),
			('filename', filename),
			('Content-Type',type),)
		ret.append(lst)
	    return ret	
	else :
	    counter = -1
	    txtcount = 0

	    for msg in self.get_loads(self) :
		counter = counter + 1
		type = msg.get_type()
	
		if type == None :
		    # do not know how to handle such attachments ...
		    # so skip them at the moment 
		    txtcount = txtcount + 1
		    continue

		if type == 'text/plain' :
		    if txtcount == 0 :
			# first txtcount is body of message
			txtcount = txtcount + 1
			continue

		# append the filename to the list
		filename = msg.get_param('name')
		if filename == None :
		    filename = "attachment %s" % counter
		lst = ( ('counter', counter), ('filename', filename),('Content-Type',type),)
		
		#'counter', counter, 'filename', msg.get_filename(),'Content-Type',type]
		ret.append(lst)
	return ret

    def get_attachement(self, number, REQUEST=None,RESPONSE=None) :
	"returns attachment number #... "
	number = int(number)
	#print "get_attachment %s" % number
	maintype = self.get_type()
	size = str(self.get_size())
	if maintype == 'application/octet-stream' :
	    RESPONSE.setHeader('Content-Type', maintype)
	    RESPONSE.setHeader('Content-Length', size)
	    RESPONSE.setHeader('Content-Disposition', 'Attachment; filename=%s' % \
			       self.get_filename())
	    return self.get_payload(decode=1) 

	
	attachment = self.get_loads(self,number)
	type = attachment.get_type()
	
	filename = attachment.get_filename()
	if type == 'message/rfc822' :
	    type = 'text/plain'
	RESPONSE.setHeader('Content-Type', type)
	RESPONSE.setHeader('Content-Length', size)
	RESPONSE.setHeader('Content-Disposition', 'Attachment; filename=%s' % filename )
	#print RESPONSE
	
	return attachment.get_payload(decode=1)

    def import_attachment(self,number,REQUEST=None,RESPONSE=None) :
	""" import attachment into zope clipboard"""
	number = int(number) 

	type = self.get_type()

	if type == 'application/octet-stream' :
	    msg = self
	else :
	    msg = self.get_loads(self,number)

	    
	# create a unique id for this object	
	id = self.create_unique_id()	
	

	if type == 'message/rfc822' :
	    type = 'text/plain'
	    filename = 'message/rfc822'
	    title = filename
	    file = str(msg)
	else :
	    filename = msg.get_param('name')
	    title = filename
	    file = msg.get_payload(decode=1)

	type = msg.get_type()
	# check images
	match = re.search('image',type)
	if match != None :
	    self.attachments.manage_addImage(id=id,file=file,title=title,content_type=type)
	else :
	    # check text / html
	    match = re.search('text',type)
	    if match != None :
		self.attachments.manage_addDTMLMethod(id,title=title,file=file)

	    else :
		# others : ad as a general file
		self.attachments.manage_addFile(id,title=title,file=file,content_type=type)
	
	msg = "attachment moved to Attachments folder"
	return self.index_html(self,REQUEST,manage_tabs_message=msg)


    def msg_body(self):
	""" returns the 'body' of the message """

	
	if self.is_multipart()  :
	   
	    for msg in self.get_loads(self) :
		type = msg.get_type()		
		if type == None :
		    type = 'text/plain'

		if type == 'text/plain' :
		    return msg.get_payload(decode=1)
	    return ""

	else :
	  	    
	    # simple mail, no attachement
	    #load = self.get_payload(decode=1)
	    return self.get_payload(decode=1)


    ####
    # for html display some data needs to be formated ...
    ###

    # html formatting functions    
    
    def change_larger_smaller(self,line) :
	" format < and > symbols to avoid confusion with html tag "
	line = string.replace(line, "<", "&lt;")
	line = string.replace(line, ">", "&gt;") 
	line = string.replace(line, '"', "&#34;") 
	return line


    def highlight_links(self,txt):
	" change appeareance of email, urls "
	ret = ""

	for line in string.split(txt,"\n") :
	    # format < and > symbols to avoid confusin with html tag 
	    line = self.change_larger_smaller(line)
	    # change emails
	    pat = re.findall("([a-zA-Z0-9\._-]*)@([a-zA-Z0-9\._-]*)",line)
	    
	    if  pat != None:
		for group in pat :
		    user  = group[0]
		    server= group[1]
		    str = "%s@%s" % (user,server)
		    # line = string.replace(line,str,'<a href="mailto:%s">%s</a>'% (str,str))
		    repstr = '<a href="folder_write_mail_html?To=%s">%s</a>' % (str,str)
		    line = string.replace(line,str, repstr)	    
	    # change urls
	    pat2 = re.search("(http://[a-zA-Z:0-9\.#_\-\/]*)",line)
	    if pat2 != None :
		url = pat2.group(1)
		line = string.replace(line,url,'<a href="%s">%s</a>' % (url,url))

	    # colors for replies
	    color1 = "#CE007B"
	    color2 = "#299C39"
	    # color replied sentences
	    pat3 = re.search('^&gt; &gt;(.*)',line)
	    if pat3 != None :
		line = '<font color="%s">%s</font>' % (color2, line)
	    else :
		pat4 = re.search('^&gt;(.*)',line)
		if pat4 != None :
		    line = '<font color="%s">%s</font>' % (color1, line)
	    
	    ret = ret + line + "\n"

	return ret

    def format_address(self,realname,address,onlyname=0) :
	""" returns a html formated text of realname, address
	if onlyname: returns only realname, if no realname, returns address """

	# format address, remove tailing character
	address = self.highlight_links(address)[:-1]
	if realname != '' :
	    if address != '' :
		if onlyname :
		    str = realname
		else :
		    str = realname + " &lt;%s&gt;" % address
	    else :
		str = realname
	else :
	    str = address
	return str

    #

    


    # the get_something functions return some html formated data
    
    def get_body(self) :
	""" returns the html formated body of the messages """
	
	type = self.get_type()
	body = self.msg_body()
	
	if type == 'application/octet-stream' :
	    # AARGH continuing would lead the server to hang!
	    
	    # return empty string, or
	    return '' 
	if self.get_type() == 'text/html' :
	    return body
	return self.highlight_links(body)
    
    def render_address_detail(self,field,part) :
	""" returns the name of a to or a from field
	part should be name or address"""

	field = string.lower(field)
	if field != 'from' and field != 'to' :
	    return "unknown field >%s<" % field
	
	email =  self[field]
	realname, address  = parseaddr(self['from'])
	if part == "name" :
	    return urllib.urlencode({'firstname':realname})
	elif part == "address" :
	    return urllib.urlencode({'email':address})
	else :
	    return "unkown part %s of email addres " % part
	
    def get_from(self,onlyname=0) :
	""" returns the sender of the email
	if onlyname == 1 returns only the name, not the address"""
	
	if self['from'] == '' :
	    return ''
	realname, address  = parseaddr(self['From'])
	str = self.format_address(realname,address,onlyname)
	return str


    def get_to(self,onlyname=0,field='To') :
	""" returns who message was sent to field allows to specify one of:
	 'To','Cc','Resent-To','Resent-Cc' """
	tos = self.get_all(field)
	
	
	# should these fields also be supported ? "
	# at some point - yes ...	
	#ccs = self.get_all('cc', [])
	#resent_tos = self.get_all('resent-to', [])
	#resent_ccs = self.get_all('resent-cc', [])
	#to_lst = getaddresses(tos + ccs + resent_tos + resent_ccs)

	
	# exception handling :
	if tos== [''] :
	    return ''
	if tos == None :
	    return None
	to_lst = getaddresses(tos)
	
	comma_flag = 0
	str = ''
	for to  in to_lst :
	    realname = to[0]
	    address  = to[1]
	    if not comma_flag :
		str = self.format_address(realname,address,onlyname)
		comma_flag = 1
	    else :
		str = str + ", " + self.format_address(realname,address,onlyname)
	if str == '':
	    # some strance address perhaps :
	    # (Recipient list suppressed)    
	    str = tos[0]
	return str

    def get_subject(self,html=1) :
	""" returns the subject of the mails """
	sbjct = self['Subject']
	if sbjct == None or sbjct == '' :
	    sbjct = "No Subject"
	if html  :
	    return self.highlight_links(sbjct)
	else :
	    return self.change_larger_smaller(sbjct)
    
	    
    def get_size(self):
	""" return the size of the email ( needed to display size in zope... """
	return self.size

    def get_date(self) :
	""" create correct date string for send mail """
	# get difference to GMT
	gmt = DateTime().toZone('GMT').strftime('%Y%m%d%H%M')
	loc = DateTime().strftime('%Y%m%d%H%M')
	d = long(loc)- long(gmt) 	
	
	if d < 0 :
	    diff = "-"
	else :
	    diff = "+"

	d = "0" + str(d)
	# if d > 9 then the leading 0 does not cause problems, \
	# since we only slice from the right:
	diff = diff + d[-4:]
	
	date = str(DateTime().strftime('%a, %d %b %Y %H:%M:%S '))
	return date + diff

    def set_values(self,
		   title='',		   
		   status='unread'  ) :
	" set title or status from outside"
		   
	self.title   = title
	self.set_status(status)

    def set_status(self,status) :
	" change the status of an email" 
	if status in self.possible_status :
	    if self.status    == 'unread' \
	       or self.status == 'queued' \
	       or status      == 'replied' \
	       or status      == 'forwarded' :	
		self.status = status 
	else :
	    raise "unknown_email_status", "unkown status: %s" % status
	
	return ""




    #####
    # send mail part 
    #####

    def send_mail(self,From,To,Subject,Body, attachment=None ,Cc=None) :
	""" send an email """
	# assume that MailHost is called MailHost :
	self.set_status('queued')

	# build upe email using the email module :
	
	self.add_header('From',From)
	self.add_header('To',To)
	if Cc != None :
	    self.add_header('Cc',Cc)
	    
	self.add_header('Subject',Subject)
	self.add_header('User-Agent','Emil_Email_Client')
	date = self.get_date()
	self.add_header('Date',date)
	    
	if attachment == None :
	    self.set_payload(Body)

	elif attachment.filename != '' :
	    # there is an attachment...
	    self.preamble = 'You will not see this in a MIME-aware mail reader.\n'
	    # To guarantee the message ends with a newline
	    self.epilogue = ''

	    msg = _Message()
	    msg.add_header('Content-Type','text/plain') 
	    msg.set_payload(Body)

	    # self.set_pauload() and self.attach() should not
	    # be used together, since
	    # since after self.set_payload() payload is not a list, byt
	    # self.attach() expect either None or [] for self._payload	    
	    self.attach(msg)
	    atta = self.create_attachment(attachment)
	    self.attach(atta)
	else :
	    # no attachment
	    self.set_payload(Body)
	
	if self.is_multipart() :
	    self.add_header('Content-Type','multipart/mixed')
	else :
	    self.add_header('Content-Type','text/plain') 

	messageText = self.as_string()
	self.MailHost.send(messageText,subject=Subject)
	self.set_status('sent')
	self.title = "%s %s" % (Subject,To)
  
    def reply_mail(self,REQUEST=None,RESPONSE=None) :
	" reply message "

	body = self.msg_body()
	new_body = "On %s, you wrote:\n" % self.Date
	for line in body.split("\n") :
	    new_body = new_body + ">" + line + "\n"
	
	first_three = self['Subject'][0:3]
	if string.lower(first_three) == "re:" :
	    # already a re: in subject...	   
	    new_subject = self.get_subject()
	else :	   
	    new_subject = "Re: %s" % self.get_subject()

	# call the write mail dtml document. ( it belongs to email folder )	
	to = self['reply-to']
	if to == None :
	    to = string.replace(self.From,'"','')
	    
	self.set_status('replied')
	if RESPONSE != None :
	    # return method of higher level email folder	   
	    url = REQUEST['URL3']
	    path = urllib.urlencode({'To':to,'Body':new_body,'Subject': new_subject})	   
	    npath = url + '/folder_write_mail_html?' + path
	    	    
	    return REQUEST.RESPONSE.redirect(npath)
	return self.folder_write_mail_html(To=to,Body=new_body,Subject=new_subject)

    def get_forward_body(self):
	""" format body of mail for forwarding """
	new_body = "\n-----Forwarded message -------\n\n"
	new_body = new_body + "From: %s\n"    % self['From']
	new_body = new_body + "To: %s\n"      % self['To']
	new_body = new_body + "Date: %s\n"    % self['Date']
	new_body = new_body + "Subject: %s\n\n" % self['Subject']
	body = self.msg_body()
	
	new_body = new_body + body
	return new_body
    
    def forward_mail(self,REQUEST,RESPONSE) :
	""" forward message """

	new_body = self.get_forward_body()	
	new_subject = "Fwd : %s" % self.get_subject()

	self.set_status('forwarded')

	if RESPONSE != None :
	    # return method of higher level email folder	    
	    url = REQUEST['URL3']	    
	    path = urllib.urlencode({'Body':new_body,'Subject': new_subject})  
	    npath = url + '/folder_write_mail_html?' + path	    	    
	    return REQUEST.RESPONSE.redirect(npath)
	
	return self.folder_write_mail_html(Body=new_body,Subject=new_subject)


   
	
    ################
    # sending of 
    # attachments
    ###############
    def headers2dictionary(self, text):
        """
        Taking text in the format of a header block convert to a dictionary
        """
	
        last_dict = None
        resultDictionary = {}
        if text != None:
	    
            for each in string.split(text, '\r\n'):
                line = string.split(each, ':')
                if len(line) > 1:
                    last_dict = string.upper(line[0])
                    resultDictionary[string.upper(line[0])] = string.join(line[1:], ':')
                elif len(line[0]) > 1 and line[0][0] in string.whitespace and last_dict is not None:
                    resultDictionary[last_dict] = resultDictionary[last_dict] + line[0][1:]

        return resultDictionary

    def create_attachment(self,attachment) :
	""" creates an attachment """
	
	head_str = str(attachment.headers)
	header = self.headers2dictionary(head_str)	  	    
	cont_type =  string.strip(header['CONTENT-TYPE'])

	#ctype = 'application/octet-stream'
	#maintype, subtype = ctype.split('/', 1)
	atta = _Message()
	atta.add_header('Content-Type',cont_type)
	
	atta.add_header('Content-Disposition', 'attachment', filename=attachment.filename)
	atta.set_payload(attachment.read())
	Encoders.encode_base64(atta)
	return atta


    # called from index_html possible actions for this message
    def email_action(self, command=None, REQUEST=None) :
	""" do something with email """

	if command == "delete" :
	    # delete email
	    self.delete_message(self.id,REQUEST)
	    message = "message deleted"
	elif command[0:5] == "move_" :
	    end_folder = command[5:]
	    parent_obj = self.REQUEST.PARENTS[0]
	    target_obj = self.resolve_path(end_folder)
	    self.move_email(parent_obj,target_obj,self.id)
	    message = "message moved to %s" % end_folder
	elif command == "spamdelete" :
	    realname, address  = parseaddr(self['from'])
	    self.add_to_spam(email=address)	    
	    self.delete_message(self.id,REQUEST)
	    message = "email %s added to spam list and deleted" % self['From'] 
	elif command == "spam" :
	    realname, address  = parseaddr(self['from'])
	    self.add_to_spam(email=address)		    
	    message = "email %s added to spam list" % self['From']	    
	else :
	    raise "Unkown_Email_Action" , command

	if REQUEST is not None :
	    m = urllib.urlencode({'manage_tabs_message':message})
	    url = REQUEST['URL2'] + "?%s" % m
	    return REQUEST.RESPONSE.redirect(url)
    
def manage_addEmail(self,id, title='',message_lst='', size=0, uid =None, REQUEST=None) :
    " add a email to Zope "

    self._setObject(id, Email(id,title,size,uid))
    
    if REQUEST is not None: return self.manage_main(self,REQUEST)




	    
	    
		    


