"""
EMIL EMAIL CLIENT
Copyright (C) 2002  Andreas Prlic
andorsch@gmx.at

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


"""
import string
import urllib

import Globals
import OFS
from OFS import SimpleItem

from Globals import DTMLFile
from Globals import Persistent
import Acquisition                	# Various acquisition types. 
import AccessControl.Role         	# Zope Security management.

import re

from email.Utils import *
from BTrees import *
import types

from pop_connection import pop_connection
from email_message import Email
from email_folder import email_folder
from addressbook import AddressBook


from filters import *

class EmailFolder(OFS.Folder.Folder) :

    """ A EMIL email client product """

    meta_type = 'EMIL_Email_Client'
    #(OFS.ObjectManager.ObjectManager.manage_options[0],)+
    manage_options = (( OFS.ObjectManager.ObjectManager.manage_options[0],)+
		      ({'label': 'Folder_Overview', 'action': 'folder_index_html',},
		       {'label': 'Write_Email', 'action': 'folder_write_mail_html',},
		       {'label': 'Check_Email', 'action': 'check_mail_html',},		      
		       {'label': 'Client_Properties',  'action': 'manage_editForm',},
		       {'label': 'Delete_Emails', 'action': 'delete_mails_html',}) +
		      AccessControl.Role.RoleManager.manage_options+
		      OFS.SimpleItem.Item.manage_options+
		      OFS.FindSupport.FindSupport.manage_options
		      )

    
    __ac_permissions__=(
	('View management screens',['manage_editForm',
				    'manage_editAction'], 	('Owner',)),
	('Emil_Send_Mail',	   ['folder_write_mail_html',
				    'email_sendAction'],	('Owner',)), 	
	('Emil_Receive_Mail',	   ['check_mail_html',
				    'check_EmailAction',
				    'delete_mails_html',
				    'delete_EmailAction'],	('Owner',)),
	('Emil_WAP_access',        ['main_wml',
				    'check_emails_wml',
				    'view_folders_wml'], 	('Owner',)),
        )     

    email_img          = '/misc_/Emil_Email_Client/email_img'
    to_email_img       = '/misc_/Emil_Email_Client/to_email_img'
    to_addressbook_img = '/misc_/Emil_Email_Client/to_addressbook_img'
    to_spam_img        = '/misc_/Emil_Email_Client/to_spam_img'
    smsfolder_img      = '/misc_/Emil_Email_Client/smsfolder_img'
    trash_img          = '/misc_/Emil_Email_Client/trash_img'

    # icons
    inbox_img     = '/misc_/Emil_Email_Client/inbox_img'
    outbox_img    = '/misc_/Emil_Email_Client/outbox_img'
    smsfolder_img = '/misc_/Emil_Email_Client/smsfilder_img'
    
    
    # The web pages that shows content. Put your own in the dtml folder.
    index_html = OFS.Folder.Folder.manage_workspace
    
    # Used to view content of the object:
    manage_editForm          = DTMLFile('dtml/emil_manage_editForm'   , globals())
    check_mail_html          = DTMLFile('dtml/check_mail_html'        , globals())
    delete_mails_html	     = DTMLFile('dtml/delete_mails_html'	      , globals())
    folder_write_mail_html   = DTMLFile('dtml/folder_write_mail_html' , globals())
    folder_index_html        = DTMLFile('dtml/folder_index_html'      , globals())

    view_folders_wml         = DTMLFile('dtml/wap/view_folders_wml'   , globals())
    folder_stat		     = DTMLFile('dtml/folder_stat'             , globals())

    # called from manage_edit
    add_filter_rule          = DTMLFile('dtml/add_filter_rule'        , globals())
    display_spam_messages    = DTMLFile('dtml/display_spam_messages'  , globals())


    # add wap pages
    main_wml                 = DTMLFile("dtml/wap/main_wml",  globals())
    check_emails_wml         = DTMLFile('dtml/wap/check_emails_wml'    , globals())
    
    def __init__ (self, id, title, server, username, password, get_new, apop='',realname='',reply_to='' ) :
	""" inits the EMIL with default values """
	
	self.id       = id
	self.title    = title
	self.server   = server
	self.username = username
	self.password = password
	self.realname = realname
	self.reply_to = reply_to
	self.spam_list = Emil_PersistentList()  # keeps track of email address that are sending spam messages. these messages are thrown away in future.
	
	# uidl keeps track of which emails have been dowloaded from server
	# it is a dictionary of email uid ( see email class)	
	# if no mails are on server, uid can be reset.
	# this should be persistent
	self.uidl = OOBTree.OOBTree() 

	# get new is a flag to download all emails, or only the new ones
	if get_new == '' :
	    self.get_new =  0
	else :
	    self.get_new = 1

	if apop == '' :
	    self.apop = 0
	else :
	    self.apop = 1
	
	#self.manage_addFolder('Outbox','outgoing email folder')
	self.manage_addEmail_Folder('Inbox','incoming email folder')
	self.manage_addEmail_Folder('Outbox','outgoing email folder')
	self.manage_addFolder('SMS','SMS Gateways')	
	self.manage_addFolder('Attachments','attachments')
	self.manage_addAddressBook('AddressBook','')
	
	# store them
	self.incoming_folder = getattr(self,'Inbox')
	self.outgoing_folder = getattr(self,'Outbox')
	self.sms_folder      = getattr(self,'SMS')
	self.attachments     = getattr(self,'Attachments')
       	self.addressbook     = getattr(self,'AddressBook')
	self.address_table   = OOBTree.OOBTree()  # persistent dictionary ...
	self.nick_table      = OOBTree.OOBTree()  # persistent dictionary ...
	
	# the filter rules to spread objects in folders ...
	self.filter_fields = ('From','To','Subject','Body')
	self.filter_rules = ()

	# set icons
	self.set_icon(self.incoming_folder,'misc_/Emil_Email_Client/inbox_img')
	self.set_icon(self.outgoing_folder,'misc_/Emil_Email_Client/outbox_img')
	self.set_icon(self.sms_folder,'misc_/Emil_Email_Client/smsfolder_img')
	
	# search for emails ...
	#new_zcatalog=context.manage_addProduct['ZCatalog'].manage_addZCatalog(id='new_id1')
	
    def manage_editAction(self, title, server, username, password, get_new='', apop='', realname='',reply_to='' , REQUEST=None,RESPONSE=None) :
	"Changes the product values"

	self.title    = title
	self.server   = server
	self.username = username
	self.password = password

	if get_new == '' :
	    self.get_new =  0
	else :
	    self.get_new = 1
	
	if apop == '' :
	    self.apop = 0
	else :
	    self.apop = 1

	self.realname = realname
	self.reply_to = reply_to

	# check icon settings
	self.set_icon(self.incoming_folder,'misc_/Emil_Email_Client/inbox_img')
	self.set_icon(self.outgoing_folder,'misc_/Emil_Email_Client/outbox_img')
	self.set_icon(self.sms_folder,'misc_/Emil_Email_Client/smsfolder_img')
	

	self._p_changed = 1
	#RESPONSE.redirect('manage_editForm')
	message = "properties have been changed"
	return self.manage_editForm(self, REQUEST,manage_tabs_message=message)


    def get_email_stats(self,obj=None):
	"return # email / folder "
	ret = []
	if obj == None :
	    obj = self
	for folder in obj.objectValues('Email_Folder') :
	    lst = folder.get_folder_stats()	    
	    ret.append(lst)
	
	return ret

    def get_folder_paths(self):
	""" returns the ids of the subfolders """
	ret = []
	for folder in self.objectValues('Email_Folder') :
	    lst = folder.get_folder_ids('')
	    for f in lst :
		ret.append(f)
	return ret

    def resolve_path(self,path):
	""" returns the folder object located at path """
	lst = string.split(path,'/')
	obj = self
	for l in lst  :
	    obj = getattr(obj,l)

	return obj
	
    def parent_from_path(self,path) :
	""" returns the parent object from a folder path """
	# get parent folder
	lst = string.split(path,"/")
	parent_path = string.join(lst[:-1],"/")
	parent_obj = self.resolve_path(parent_path)
	return parent_obj
    
    def get_emil_stats(self,obj) :
	""" get statistics of subfolders """

	folder_lst = []
	start_depth = 0
	max_depth   = 0
	for folder in obj.objectValues('Email_Folder') :
	    data,depth = folder.get_sub_stats(start_depth+1)
	    if depth > max_depth :
		max_depth = depth
	    folder_lst.append(data)
	return max_depth,folder_lst

    def _render_folder_data(self,data,maxdepth,depth,hilite,path=None):
	""" subfunction of render_emil_stats"""
	if hilite :
	    cl = 'class="row-hilite"'
	else :
	    cl = 'class="row-normal"'
	#html ="%s %s\t" % (maxdepth, depth)
	html = ''

	if path == None :
	    path = str(data['id'])
	else :
	    path = path + "/" + str(data['id'])
	    
	url     = '<a href="%s">%s</a>' % (path,data['id'])
	total   = '<center><b>%s</b></center>' % (data['total'])
	unread = '<center><font color="#5A5E25"><b>%s</b></font></center>' % (data['unread'])
	#checkbutton = '<td><input type="checkbox" name="paths:list" value="%s"></td>' % (path)
	checkbutton = ""
	
	if maxdepth-depth == 1 :
	    html += '<tr %s>%s<td>%s</td><td colspan="%s">&nbsp;</td><td>%s</td><td>%s</td></tr>\n' \
		   % (cl,checkbutton,url,depth,total,unread)
	    
	elif depth == 1 :
	    html += '<tr %s>%s<td colspan="%s">&nbsp;</td><td>%s</td><td>%s</td><td>%s</td></tr>\n' \
		   % (cl,checkbutton,maxdepth-depth, url,total,unread)
	else :
	    html += '<tr %s>%s<td colspan="%s">&nbsp;</td><td>%s</td><td colspan="%s">&nbsp</td><td>%s</td><td>%s</td></tr>\n' \
		   % (cl,checkbutton,maxdepth-depth, url,depth-1,total,unread)

	# subfolders ...

	for subdat in data['sublist'] :
	    if hilite :
		hilite = 0
	    else :
		hilite = 1
	    h,hilite = self._render_folder_data(subdat,maxdepth,depth-1,hilite,path)
	    html += h
	   
	return html,hilite
	    
    def render_emil_stats(self,obj=None):
	""" make html display of emil stats, start at folder obj"""

	if obj == None :
	    obj = self
	depth,data = self.get_emil_stats(obj)
	if depth == 0 :
	    return ""

	html = ""
	#html = '<form action="." method="get">'
	html += "<table border=0>\n"	
	hilite = 1
	html += '<tr class="list-header">\n'
	html += '<td colspan="%s" NOWRAP><div class="list-item">Subfolders of <b>%s</b></div></td>\n' % (depth + 2,obj.id)
	html += '<td><div class="list-item">total</div></td>\n'
	html += '<td><div class="list-item">unread</div></td></tr>\n'
	for folder in data :
	    if hilite :
		hilite = 0
	    else :
		hilite = 1
	    h,hilite = self._render_folder_data(folder,depth+2,depth+1,hilite)
	    html += h

	html += "</table>"
	#html += '<input class="form-element" type="submit" name="rename_folder:method" value="Rename Folder" >'
	#html += '<input class="form-element" type="submit" name="delete_folder:method" value="Delete Folder" >'
	#html += '</form>'
	#print html
	return html

    def rename_folder(self,paths,REQUEST):
	""" rename a folder """
	# only do for first in selection
	path = paths[0]
	folder_obj = self.resolve_path(path)

	parent_obj = self.parent_from_path(path)
	lst =string.split(path,"/")
	p = string.join(lst[:-1],"/")
	il = "%s" % folder_obj.id
	ids = urllib.urlencode({'ids':il})
	url=REQUEST['URL1'] + "/" + p + '/manage_renameForm?' + ids
	
	lst = string.split(ids,"=")
	pa = string.join(lst[1:],"=")

	self.REQUEST.set('ids',[pa])

	
	return REQUEST.RESPONSE.redirect(url)		    
    
    def rename_subfolder(self,id):
	""" rename a subfolder """
	ids = [id,]
	return self.manage_renameForm(ids=ids,REQUEST=REQUEST)
    
    def delete_folder(self,paths=None,REQUEST=None) :
	""" delete folders that are empty"""

	for path in paths :
	    folder_obj = self.resolve_path(path)

	    # check if folder is empty
	    lst = folder_obj.objectItems()
	    if lst != [] :
		message = "folder <b>%s</b> is not empty!" % path
		return self.folder_index_html(self, REQUEST,manage_tabs_message=message)

	    # check if it is used in any filters

	    for filter in self.filter_rules :
		if filter['folder'] == path :
		    
		    message = "folder <b>%s</b> can not be deleted, is used in filter rule %s" % (path,filter['title'])
		    return self.folder_index_html(self, REQUEST,manage_tabs_message=message)
		    
		    
		    

	    # folder is empty, can be deleted

	    # get parent folder	   
	    parent_obj = self.parent_from_path(path)

	    # and delete folder
	    parent_obj.manage_delObjects(folder_obj.id,REQUEST=REQUEST)
	    
	message = "folder deleted"
	return self.folder_index_html(self, REQUEST,manage_tabs_message=message)
    
    
    def reset_uidl(self) :
	" reset the uid dictionary, only do this when there are no mails on server... "
	self.uidl = OOBTree.OOBTree() 

    def open_connection(self) :
	""" returns open connection """
	pop = pop_connection(server=self.server,user=self.username,password=self.password,apop=self.apop)
	pop.open()

	return pop
	
    def check_EmailAction(self,return_flag=0,REQUEST=None) :
	"""checks the email
	return_flag can be 0 for no return,
	'index' for returning index_html
	or 'sms' for returning a message"""
	
	pop = self.open_connection()
	
	number_mails, octet =  pop.status()	
	# no mails on server, uidl can be reset
	if number_mails == 0 :
	    self.reset_uidl()
	    message= "no mails on server"
	    return self.folder_index_html(self, REQUEST,manage_tabs_message=message)

	messages = pop.get_messages(get_new=self.get_new,known_uidl=self.uidl)
	pop.close()

	txt = ""
	
	c = 0
	for message in messages :	    
	    txt  = message['list']
	    size = message['size']
	    uid  = message['uid']
	    id = self.create_unique_id()
	    
	    #id = "%s" % c 
	    title = ''

	    self.incoming_folder.manage_addEmail(id,title,size,uid)
	    obj = getattr(self.incoming_folder,id)
	    obj.from_server_list(txt)

	    # check if this email is a spam email!
	    # if it is - remove it again
	    realname,sender = parseaddr(obj['from'])	   
	    if self.in_spam_list(sender) :		
		# skip it !
		self.incoming_folder.manage_delObjects(id,REQUEST=REQUEST)		
		self.uidl[uid] = 1
		continue
	    
	    c = c + 1
	    # apply filter rules :
	    self.apply_filter_rules(obj)

	    self.uidl[uid] = 1

	    
	
	if c > 1:
	    message = "downloaded <b>%s new mails</b> (on server: %s)" % (c,number_mails)
	elif c == 1 :
	    message = "downloaded <b>1 mail</b> (on server: %s)" % number_mails
	else :
	    if number_mails == 1 :
		message = "no new mails on server (%s old mail)" % number_mails
	    else :
		message = "no new mails on server (%s old mails)" % number_mails

	if return_flag == 0 :
	    return
	elif return_flag == 'index' :
	    return self.folder_index_html(self, REQUEST,manage_tabs_message=message)
	elif return_flag == 'sms' :
	    return message
	else :
	    msg = "unknown return_flag %s" + return_flag
	    raise msg


    def get_fields_from_header(self,header):
	""" takes the header of a message as a list, returns afield """
	fr = ('fr',re.compile(r'^from: '   ,re.IGNORECASE))
	to = ('to',re.compile(r'^to: '     ,re.IGNORECASE))
	su = ('su',re.compile(r'^subject: ',re.IGNORECASE))

	res = [ fr, to, su ]
	su=fr=to= ''

	for line in header :
	    for reg in res :

		obj =  reg[1].search(line)
		if obj != None:
		    line = string.replace(line,"'","")
		    cmd = "%s = '%s'" % (reg[0],line)
		    exec cmd
	return fr,to,su
    
    def server_list(self): 
	""" returns list of messages on server """

    	pop = self.open_connection()

	zlst = ()
	
	list =  pop.list()
	summary = list[0]
	for l in list[1] :
	    number = string.split(l)[0]
	    try:
		top = pop.top(number)
	    except:
		message= "your mail server does not support the command top, could not delete messages!"
	    	return self.folder_index_html(self, REQUEST,manage_tabs_message=message)

	    fr,to,su = self.get_fields_from_header(top[1])
	    #print l,fr,to,subject
	    obj = {}
	    obj['pos'] = number
	    obj['fr']  = fr 
	    obj['to']  = to 
	    obj['su']  = su 
	    zlst += (obj,)

	pop.close()
	#print list
	return zlst

    def delete_EmailAction(self,ids=None,REQUEST=None):
	""" manage the delete email menu """
	
	pop = self.open_connection()
	for id in ids :
	    # delete message nr. id from server
	    pop.delete(int(id))
	pop.close()
	message = "deleted %s messages from server" % len(ids)
	
	return self.folder_index_html(self, REQUEST,manage_tabs_message=message)

	
    def move_email(self,start_obj,end_obj, id_message) :
	""" move an email from start_obj to end_obj """
	#print "moving %s from %s to %s" % (id_message, start_obj.id, end_obj.id)
	ret = start_obj.manage_cutObjects(id_message)
	end_obj.manage_pasteObjects(ret)
	
	
    def create_unique_id(self) :
	" create unique id"

	from DateTime import DateTime
	import random
	id = str(DateTime().millis())
	
	id = id + str(random.randrange(100000))
	
	return id

    def email_sendAction(self,To='',Cc='',From='',Subject='',Body='',REQUEST=None) :
	" send an email "
	id     = self.create_unique_id()
	title  = ''
	uid    = None
	size   = 0
	
	self.outgoing_folder.manage_addEmail(id,title,size,uid)
	
	obj = getattr(self.outgoing_folder,id)

	if REQUEST is not None :
	    attachment = REQUEST.form['attached_file']
	else :
	    attachment = None

	# check for old versions which do not have nick_table
	try:
	    nick = self.nick_table
	except:
	    self.address_table   = OOBTree.OOBTree()
	    self.nick_table      = OOBTree.OOBTree()
	    
	to_str = self.replace_nicknames(To)
	cc_str = self.replace_nicknames(Cc)

	if cc_str == '' :
	    obj.send_mail(From=From, To=to_str, Subject=Subject,Body=Body, attachment=attachment )
	else :
	    obj.send_mail(From=From, To=to_str, Subject=Subject,Body=Body, attachment=attachment,Cc=cc_str )
	message = "message successfully sent to <b>%s</b>" % To
	
	return self.folder_index_html(self, REQUEST,manage_tabs_message=message)


    def get_addressbook(self):
	""" returns the addressbook object """
	abook = getattr(self,'AddressBook')
	return abook
    
    def replace_nicknames(self,To) :
	""" exchange nichnames against real address name """

	# replace nicknames
	spl = string.split(To,",")
	to_str = ""	
	for name in spl :
	    name = string.strip(name)
	    # check if name in addressbook nicknames
	    if self.nick_table.has_key(name) :
		abook = self.get_addressbook()
		address = getattr(abook,self.nick_table[name])
		name = address.__str__()

	    if to_str == '' :
		to_str = name
	    else :
		to_str = to_str + ", " + name
	
	return to_str
    
    def manage_addEmail_Folder(self,id,title):
	" add an email folder"	
	self._setObject(id,email_folder(id,title))

    def manage_addAddressBook(self,id,title) :
	""" add an addressbook """
	self._setObject(id,AddressBook(id,title))

	
    # Filter rules

    def apply_filter_rules(self, email) :
	" return the path of the folder to move the email into ..."

	filter_manager = FilterManager(self.filter_rules,self.incoming_folder)
	filter_manager.apply(email,sms=1)
	
	
    def add_filter_action(self,field,querytext,comparison,action,folder=None,forwardto=None,executemethod=None,toaddressbook=None,groups=None,REQUEST=None) :
	" add a new filter rule "

	title =  "<strong>%s</strong> %s <strong>%s</strong> : " % ( field, comparison , querytext )

	#for act in ['move','delete','forward','execute','toaddressbook' ] :
	if type(action) == types.StringType :
	    action = [action,]
	for act in action :
	    if act[0:4] == 'move' :
		title += "<strong>move</strong> to %s, " % folder
	    elif act == 'delete' :
		title += '<strong>delete</strong>, '
	    elif  act == 'forward' :
		title += '<strong>forward</strong> to %s,' % forwardto
	    elif act == 'execute' :
		title += '<strong>execute</strong> method <b>%s</b>' % executemethod
	    elif act == 'toaddressbook' :
		title += ' <strong>move to addressbook</strong> ( add to groups : %s) ' % groups
	    
	
	
	#title = "%s contains %s move to %s" %(field,querytext,folder)
	rule = {}

	for index in [ 'title', 'action', 'field','querytext','comparison','folder','forwardto','executemethod','toaddressbook','groups' ] :
	    cmd = "rule[\'%s\'] = %s" % (index,index)
	    exec cmd

	    

	t =(rule,)
	self.filter_rules = self.filter_rules + t
	self._p_changed = 1

	if REQUEST is not None :
	    message = "filter rule has been added"	
	    return self.manage_editForm(self, REQUEST,manage_tabs_message=message)
	
    def delete_filter_rule(self,number,REQUEST=None) :
	""" delte a filter rule """
	number = int(number)
	ts =self.filter_rules[:number]
	te =self.filter_rules[number+1:]
	self.filter_rules = ts + te

	if REQUEST is not None :
	    message = "filter rule has been deleted"	
	    return self.manage_editForm(self, REQUEST,manage_tabs_message=message)


    # deal with spam stuff

    def get_spam_messages(self):
	""" returns the spam emails """
	try:
	    keys = self.spam_list.keys()
	except:
	    # check for old versions which do not have spam_list
	    self.spam_list = Emil_PersistentList()
	    keys = self.spam_list.keys()
	return keys
    
    def add_to_spam(self,email,REQUEST=None) :
	""" report an email address as a spam address.
	emails from this address will be thrown away."""
	try:
	    self.spam_list.append(email)
	except:
	    # check for old versions which do not have spam_list
	    self.spam_list = Emil_PersistentList()
	    self.spam_list.append(email)
	    
	get_transaction().commit()
	
	if REQUEST is not None :
	    message = "email %s added to spamlist" % email
	    return self.display_spam_messages(self, REQUEST,manage_tabs_message=message)

    def delete_from_spam(self,email,REQUEST=None):
	""" removes an email from the spam list"""
	self.spam_list.delete(email)	
	get_transaction().commit()
	if REQUEST is not None :
	    message = "email %s removed from spamlist" % email
	    return self.display_spam_messages(self, REQUEST,manage_tabs_message=message)

    def in_spam_list(self,email) :
	""" return 1 if email in spam_list """
	try:
	    inlist = self.spam_list.in_spam_list(email)
	except:
	    # check for old versions which do not have spam_list
	    self.spam_list = Emil_PersistentList()
	    return 0
	return inlist



    # management of address - nicknames
    def update_address_table(self) :
	""" store nicknames of all entries in addressbook in memory """
	
	# initialize self.address_table   = OOBTree.OOBTree()
	self.address_table   = OOBTree.OOBTree()
	self.nick_table      = OOBTree.OOBTree()
	
	addressbook = getattr(self,'AddressBook')
	items = addressbook.objectItems('Emil_address')
	
	for id,obj in items :
	    nickname = obj.get_nickname()
	    if nickname != '' :
		if self.nick_table.has_key(nickname) :
		    raise "not_unique_Nickname", "nickname %s given twice!"  % nickname
		self.nick_table[nickname] = id

	    email = obj.get_email()
	    if email != '' :
		self.address_table[email] = id
		
    def in_address_table(self,sender):
	""" check if a given email has a nick in the addressbook"""
	if sender == "" :
	    return 0

	try:
	    # check for old versions which do not have address_table
	    table = self.address_table
	    
	except:
	    self.address_table = OOBTree.OOBTree()
	    self.nick_table      = OOBTree.OOBTree()
	    return 0
	
	mat = re.search("([a-zA-Z0-9\._-]*)@([a-zA-Z0-9\._-]*)",sender)	
	user  = mat.group(1)
	server= mat.group(2)
	email = "%s@%s" % (user,server)	
	if self.address_table.has_key(email) :
	    return 1
	else :
	    return 0


    # helper functions
    def urlencode(self,name,str):
	return urllib.urlencode({name:str})

    def set_icon(self,obj,iconname):
	obj.icon = iconname
	obj._p_changed = 1


# constructor pages. Only used when the product is added to a folder:

# Get user id from this form :
manage_addEMILForm 	= DTMLFile('dtml/emil_manage_addForm', globals()) 

    
def manage_addEMILAction(self,
			 id='EMIL',
			 title='Title here',
			 server='',
			 username='',
			 password='',
			 get_new='',
			 apop='',
			 realname='',
			 reply_to='',			 
			 REQUEST=None):
    "Add a EMIL"
    self._setObject(id, EmailFolder(id, title, server, username, password,get_new,apop,realname,reply_to))
   
    if REQUEST is not None:
	return self.manage_main(self, REQUEST)



    
class Emil_PersistentList(Persistent) :
    """ a structure to store email data in zodb """
    def __init__(self):
	""" """
	
	# would be nice to have a real persistent list for doing this.
	# I tried to create one but did not work properly.
	# I am not experienced with Persistence stuff and ZODB :-/
	# so this is a workaround. - But it works
	# if you have a nicer solution - send me a patch :-)
	
	self.spams = OIBTree.OIBTree()

    def append(self,email) :
	self.spams[email] = 1

    def keys(self) :
	return self.spams.keys()

    def delete(self,email) :
	del self.spams[email]
	
    def in_spam_list(self,email) :
	if self.spams.has_key(email) :
	    return 1
	else :
	    return 0

    
	




