"""
EMIL EMAIL CLIENT
Copyright (C) 2002  Andreas Prlic
andorsch@gmx.at

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


"""

import string

from email.Utils import *
from addressbook import manage_addAddress

# this file takes care of the filter handling

class FilterManager :
    """ manages the action of filter rules """
    filter_fields = ('From','To','Subject','Body')
    
    def __init__(self,filter_rules,path) :
	""" initialize the rules """

	self.filter_rules = filter_rules
	self.path = path
	
    def apply(self,email,sms=0) :
	""" apply filter rules on this email """

	folder_obj = self.path
	# go through all rules
	for rule in self.filter_rules :

	    try:
		actions = rule['action']
	    except :
		# catch for old filters :
		rule['action'] = ['move',]
		actions = rule['action']
		
	    for action in actions :
		
		if action == 'move' :
		    # sometimes two filter affect the same message, then this would
		    # cause an attribute error, therefore catch it.
		    try:
			folder_obj = self.move_action(rule,email)
		    except:
			print "unable to move email %s according to rule %s" % (email, rule['title'])

		elif action == 'forward' :
		    body = email.get_forward_body()
		    subject = "Fwd : %s" % email.get_subject()
		    fr = email['to']
		    
		    self.path.email_sendAction(To=rule['forwardto'],From=fr,Subject=subject,Body=body)
		    email.set_status('forwarded')
		    
		
		elif action == 'toaddressbook' :
		    name,address=parseaddr(email['from'])
		    abook = self.path.get_addressbook()
		    manage_addAddress(abook,firstname=name,email=address)
		    
		elif action == 'execute' :
		    obj = getattr(self.path,rule['executemethod'])
		    ret = obj(email)
		    
		elif action == 'delete' :
		    self.path.delete_message(email.id)
		else :
		    print "action >%s< no supported so far ..." % action
	if sms :
	    folder_obj.check_send_sms(email)	
	
    def move_action(self,rule,email) :
	""" move action returns folder object where email has been moved to"""

	for field in self.filter_fields :
	    if field != rule['field'] : continue 
	    if field == 'Body' :
		searchtext = email.get_body()
	    else :
		searchtext= email[field]
		
	    #print email['subject']
	   
	    #print ">%s< in >%s< ?" % (searchtext,rule['querytext'])
	    res = string.find(searchtext,rule['querytext'])
	    #print res
	    if res != -1 :
		# found !		
		# move obj to new path
		start_obj = self.path
		# set changed folder
		target_obj = self.path.resolve_path(rule['folder'])
		#print "move %s from %s to %s " % ( email.id,start_obj.id, target_obj.id)
		self.path.move_email(start_obj,target_obj,email.id)		
		return target_obj
	
	return self.path

	
